'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

var getPreviousEnabled = function getPreviousEnabled(currentPage) {
  return currentPage > 0;
};

var getNextEnabled = function getNextEnabled(currentPage, totalPages) {
  return currentPage + 1 < totalPages;
};

var getTotalPages = function getTotalPages(totalItems, pageSize) {
  return Math.ceil(totalItems / pageSize);
};

var getStartIndex = function getStartIndex(pageSize, currentPage) {
  return pageSize * currentPage;
};

var getEndIndex = function getEndIndex(pageSize, currentPage, totalItems) {
  var lastPageEndIndex = pageSize * (currentPage + 1);

  if (lastPageEndIndex > totalItems) {
    return totalItems - 1;
  }

  return lastPageEndIndex - 1;
};

var getPaginationState = function getPaginationState(_ref) {
  var totalItems = _ref.totalItems,
      pageSize = _ref.pageSize,
      currentPage = _ref.currentPage;
  var totalPages = getTotalPages(totalItems, pageSize);
  return {
    totalPages: totalPages,
    startIndex: getStartIndex(pageSize, currentPage),
    endIndex: getEndIndex(pageSize, currentPage, totalItems),
    previousEnabled: getPreviousEnabled(currentPage),
    nextEnabled: getNextEnabled(currentPage, totalPages)
  };
};
function usePagination(_ref2) {
  var _ref2$totalItems = _ref2.totalItems,
      totalItems = _ref2$totalItems === void 0 ? 0 : _ref2$totalItems,
      _ref2$initialPage = _ref2.initialPage,
      initialPage = _ref2$initialPage === void 0 ? 0 : _ref2$initialPage,
      _ref2$initialPageSize = _ref2.initialPageSize,
      initialPageSize = _ref2$initialPageSize === void 0 ? 0 : _ref2$initialPageSize;

  var _React$useState = React.useState(initialPageSize),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      pageSize = _React$useState2[0],
      setPageSize = _React$useState2[1];

  var _React$useReducer = React.useReducer(function (state, action) {
    switch (action.type) {
      case "SET":
        return action.page;

      case "NEXT":
        if (!getNextEnabled(state, getTotalPages(totalItems, pageSize))) {
          return state;
        }

        return state + 1;

      case "PREV":
        if (!getPreviousEnabled(state)) {
          return state;
        }

        return state - 1;

      default:
        return state;
    }
  }, initialPage),
      _React$useReducer2 = _slicedToArray(_React$useReducer, 2),
      currentPage = _React$useReducer2[0],
      dispatch = _React$useReducer2[1];

  var paginationState = React.useMemo(function () {
    return getPaginationState({
      totalItems: totalItems,
      pageSize: pageSize,
      currentPage: currentPage
    });
  }, [totalItems, pageSize, currentPage]);
  return _objectSpread({
    setPage: React.useCallback(function (page) {
      dispatch({
        type: "SET",
        page: page
      });
    }, [dispatch]),
    setNextPage: React.useCallback(function () {
      dispatch({
        type: "NEXT"
      });
    }, [dispatch]),
    setPreviousPage: React.useCallback(function () {
      dispatch({
        type: "PREV"
      });
    }, [dispatch]),
    setPageSize: React.useCallback(function (pageSize) {
      var nextPage = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      setPageSize(pageSize);
      dispatch({
        type: "SET",
        page: nextPage
      });
    }, [setPageSize]),
    currentPage: currentPage,
    pageSize: pageSize
  }, paginationState);
}

function Pagination(_ref3) {
  var children = _ref3.children,
      _ref3$totalItems = _ref3.totalItems,
      totalItems = _ref3$totalItems === void 0 ? 0 : _ref3$totalItems,
      _ref3$initialPage = _ref3.initialPage,
      initialPage = _ref3$initialPage === void 0 ? 0 : _ref3$initialPage,
      initialPageSize = _ref3.initialPageSize;
  return children(usePagination({
    totalItems: totalItems,
    initialPage: initialPage,
    initialPageSize: initialPageSize
  }));
}

Pagination.displayName = "Pagination";

exports.getPaginationState = getPaginationState;
exports.usePagination = usePagination;
exports.default = Pagination;
